#ifndef __FS_h__
#define __FS_h__

#include "fs/FSAttributed.H"
#include "fs/FSResult.H"

class FSDirImpl;

/**
 * This class provieds an interface to access any filesystem. The methods only
 * deal with <strong>global</strong> operations such as filesystem checking. For
 * <strong>local</strong> operations such as creating or reading files different
 * classes are given.
 *
 * @see FSObj
 * @see FSDir
 * @see FSFile
 */
class FS : public FSAttributed
{
public:
  /** Definition of a type used to count numbers of bytes */
  typedef unsigned long Count;
  /** Definition of a type used to store a position in a file */
  typedef unsigned long Pos;

  virtual ~FS () {}

  /**
   * Checks if the filesystem is working. This includes checks of the
   * medium used and the structures of the filesystem.
   *
   * @return Returns the status of the operation as a FSResult object.
   */
  virtual FSResult check() = 0;

  /**
   * Get access to the root directory of the filesystem.
   *
   * @return Retruns the directory implementation of the filesystem.
   */
  virtual FSDirImpl *root() = 0;

  /**
   * Stores all buffered data to get a consistant state on the medium.
   *
   * @return Returns the status of the operation as a FSResult object.
   */
  virtual FSResult sync() = 0;
};

#endif // __FS_h__
