#ifndef __FSDir_h__
#define __FSDir_h__

#include "fs/FSObj.H"

class FS;
class FSFile;
class FSDirImpl;

/**
 * This class is a direcotry implementation that interacts with the application
 * operations on the filesystem are done by a filesystem specific implementation
 * of the interface FSDirImpl.
 */
class FSDir : public FSObj
{
public:
  /**
   * This constructor creates an uninitialized directory object.
   */
  FSDir();

  /**
   * This constructor creates a references to the root directory of the given 
   * filesystem.
   *
   * @param filesystem instance of the filesystem
   */
  FSDir(FS* filesystem);

  /**
   * This method identifies this object as a directory
   *
   * @return Returns the type of the object identifying it as a directory.
   */
  Type type () { return Directory; }

  /**
   * Reads one entry from the directory listing.
   *
   * @param buffer pointer to the buffer used to store the entry data
   * @param buffer_size size of the entry data (will be modified by operation)
   * @param type type of the entry (will be modified by operation)
   *
   * @return Returns the status of the operation as a FSResult object.
   */
  FSResult read_entry (char* buffer, int &buffer_size, FSObj::Type &type);
  FSResult reset ();

  /**
   * Gets a filesystem object (<em>directory</em>) from this directory.
   * 
   * @param name name of the filesystem object
   * @param dir instance of a directory object (will be modified by operation)
   *
   * @return Returns the status of the operation as a FSResult object.
   */
  FSResult get_entry (const char* name, FSDir &dir);

  /**
   * Gets a filesystem object (<em>file</em>) from this directory.
   * 
   * @param name name of the filesystem object
   * @param file instance of a file object (will be modified by operation)
   *
   * @return Returns the status of the operation as a FSResult object.
   */
  FSResult get_entry (const char* name, FSFile &file);
  
  /**
   * Creates an entry for a given filesystem object (<em>directory</em>) in 
   * this directory.
   * 
   * @param name new name of the filesystem object
   * @param dir instance of a directory object
   *
   * @return Returns the status of the operation as a FSResult object.
   */
  FSResult mkdir (const char* name, FSDir &dir);
  
  /**
   * Creates an entry for a given filesystem object (<em>file</em>) in this 
   * directory.
   * 
   * @param name new name of the filesystem object
   * @param file instance of a file object
   *
   * @return Returns the status of the operation as a FSResult object.
   */
  FSResult create (const char* name, FSFile &file);
};

#endif // __FSDir_h__
