#ifndef __FSFile_h__
#define __FSFile_h__

#include "fs/FS.H"
#include "fs/FSObj.H"

/**
 * This class is a file implementation that interacts with the application.
 * Operations on the filesystem are done by a filesystem specific implementation
 * of the interface FSFileImpl.
 */
class FSFile : public FSObj
{
public:
  /**
   * This method identifies this object as a file.
   *
   * @return Returns the type of the object identifying it as a file.
   */
  Type type () { return File; }

  /**
   * Reads data from the file to a buffer.
   *
   * @param buffer buffer to read to
   * @param buffer_size size of the buffer
   * @param pos start position in the file
   * @param really_transfered number of bytes read 
   * @return Returns the error status of the filesystem as an FSResult instanze 
   *         (default is FSResult::NoError).
   */
  FSResult read(void* buffer, FS::Count buffer_size, FS::Pos pos, FS::Count &really_transfered);

  /**
   * Writes a number of bytes from a buffer to the file.
   * 
   * @param buffer contains the data that will be written
   * @param bytes_to_write number of bytes to write
   * @param pos start position in the file
   * @return Returns the error status of the filesystem as an FSResult instanze 
   *         (default is FSResult::NoError).
   */
  FSResult write(void* buffer, FS::Count bytes_to_write, FS::Pos pos);

  /**
   * Truncates the file to the given length.
   *
   * @param pos new max. lenght of the file (if pos is bigger than the actual 
   *        size of the file nothin is done.
   * @return Returns the error status of the filesystem as an FSResult instanze 
   *         (default is FSResult::NoError).
   */
  FSResult trunc(FS::Pos pos);
};

#endif // __FSFile_h__
