#ifndef __FSFileImpl_h__
#define __FSFileImpl_h__

#include "fs/FS.H"
#include "fs/FSObjImpl.H"

/**
 * Interface for the implementation of a filesystem specific file.
 */
class FSFileImpl : public FSObjImpl
{
public:
  FSFileImpl(FS *filesystem) : FSObjImpl(filesystem) {}
  virtual ~FSFileImpl() {}

  /**
   * Reads data from the file to a buffer.
   *
   * @param buffer buffer to read to
   * @param buffer_size size of the buffer
   * @param pos start position in the file
   * @param really_transfered number of bytes read 
   * @return Returns the error status of the filesystem as an FSResult instanze 
   *         (default is FSResult::NoError).
   */
  virtual FSResult read(void* buffer, FS::Count buffer_size, FS::Pos pos, FS::Count &really_transfered) = 0;

  /**
   * Writes a number of bytes from a buffer to the file.
   * 
   * @param buffer contains the data that will be written
   * @param bytes_to_write number of bytes to write
   * @param pos start position in the file
   * @return Returns the error status of the filesystem as an FSResult instanze 
   *         (default is FSResult::NoError).
   */
  virtual FSResult write(void* buffer, FS::Count bytes_to_write, FS::Pos pos) = 0;

  /**
   * Truncates the file to the given length.
   *
   * @param pos new max. lenght of the file (if pos is bigger than the actual 
   *        size of the file nothin is done.
   * @return Returns the error status of the filesystem as an FSResult instanze 
   *         (default is FSResult::NoError).
   */
  virtual FSResult trunc(FS::Pos pos) = 0;
};

#endif // __FSFile_h__
