#ifndef __FSObj_h__
#define __FSObj_h__

#include "fs/FSResult.H"

class FS;
class FSObj;
class FSDir;
class FSObjImpl;
class FSDirImpl;
class FSFileImpl;

class FSObj
{
  /** name of the object is provided by the filesystem client */
  const char *name;
  unsigned char itsmybuffer;
  /** corresponding filesystem object, 0 if not initialized */
  FSObjImpl *impl;
protected:
  /** directory where this object is located, 0 for the root dir object */
  FSDirImpl *dir;

  FSResult check_impl();
  FSResult check_dir();
  FSDirImpl *impl_dir() { return (FSDirImpl*)impl; }
  FSFileImpl *impl_file() { return (FSFileImpl*)impl; }
public:
  FSObj() : name (0), itsmybuffer(0), impl (0) {}
  FSObj(const char* obj_name) : name (obj_name), itsmybuffer(0), impl (0) {}
  FSObj(const FSObj& copy) 
  { 
    init(copy);
  }
  FSObj& operator = (const FSObj& copy) 
  {
    release();
    init(copy);
    return *this; 
  }
  virtual ~FSObj() { release (); }

  /** public types of filesystem objects */
  typedef enum { Directory, File, Unknown } Type;
  virtual Type type() { return Unknown; }

  // Methods to connect to and disconnect from a filesystem object
  void release();
  void init(const FSObj& copy);
  void init(const char* obj_name, FSDirImpl *directory, FSObjImpl *object);

  /**
   * Creates a new hardlink for this filesystem object.
   * 
   * @param location interface of the destination directory
   * @param link_name name of the link
   * @return Returns the error status of the filesystem as an FSResult instanze 
   *         (default is FSResult::NoError).
   */
  FSResult link(FSDir& location, const char* link_name);

  /**
   * Moves this filesystem object to a new location.
   *
   * @param location interface of the destination directory
   * @param new_name new name of the entry
   * @return Returns the error status of the filesystem as an FSResult instanze 
   *         (default is FSResult::NoError).
   */
  FSResult move(FSDir& location, const char* new_name);

  /**
   * Removes this filesystem object by releasing the allocated structures.
   *
   * @return Returns the error status of the filesystem as an FSResult instanze 
   *         (default is FSResult::NoError).
   */
  FSResult remove();

  FSResult attribute_count (int &count);
  FSResult attribute_name (int n, const char *&attr_name);
  FSResult attribute_description (int n, const char *&description);
  FSResult set_attribute (const char* attr_name, const char *val);
  FSResult get_attribute (const char* attr_name, char *val, int& size);
};
      
#endif // __FSObj_h__
