#ifndef __FSResult_h__
#define __FSResult_h__

/**
 * This class contains the codes and messages of the errors that can occur using
 * a filesystem.
 */
class FSResult
{
public:
  typedef enum 
  {
     NoError,          // no error
     EndOfFile,        // end of file (not realy an error)
     FsSpecificError,  // unknown error of the filesystem
     FileNotFound,     // :-)
     PermissionDenied, // missing permission
     DiskFull,         // no more space left on device
     AttribUnknown,    // unknown attribute
     IOError,          // 
     NoMedium,         // no disk in drive
     InvalidFS,        // wrong filesystem on device
     InvalidForRoot,   // operation not allowed
     UninitializedObj, // uninitialised object
     ResourceLimit,    // limit of resourced reached
     BufferTooSmall,   // given buffer is too small
     NoDirectory,      // given object is no directory
     NoFile,           // given object is no file
     NotSupported,     // :-)
     Readonly,         // trying to write a read-only filesystem
     NotChangeable,    // try to change a non changeble attribute
     InvalidValue,     // try to set an attribute to an invalid value
     ObjBusy,          // try to delete an used object
     NotEmpty,         // try to delete a directory that is not empty
     OwnSubstructure   // access to the own substructure
  } Code;

private:
  Code _code;
public:
  FSResult () : _code (NoError) {}
  FSResult (Code errorcode): _code (errorcode) {}

  /**
   * Gives an error message for an occured error.
   * 
   * @return Returns information on the occured error.
   */
  const char* message () const;

  // compare error codes
  bool operator == (Code error) { return (_code == error); }
  bool operator != (Code error) { return (_code != error); }
  bool operator == (FSResult error) { return (_code == error._code); }
  bool operator != (FSResult error) { return (_code != error._code); }

  /**
   * Checks if there was an error.
   *
   * @return Returns true if there was no error.
   */
  operator bool () { return (_code == NoError); }
};

#endif // __FSResult_h__
