#ifndef __Unix_Device_h__
#define __Unix_Device_h__

#include "fs/device/UnixDeviceLogic.H"
#include "fs/device/block/BlockDevice.H"
#include "system/Mutex.H"

// This class implements the standard interface for block oriented devices
// on a UNIX file device. It also implements the necessary locking for
// multithreaded device access.
class UnixDevice : public UnixDeviceLogic, public virtual BlockDevice
{
  // mutex variable to ensure consistency of the driver state information
  Mutex mutex;

  // stores the allowed access type
  enum AccessType { RW, R, NONE } access;
public:
  UnixDevice (const char* name);
  ~UnixDevice ();

  // select the preferred logical block size
  BlockDeviceError setup (BlockSize block_size);

  // get device characteristics
  BlockSize block_size ();
  BlockCount block_count ();

  // read and write multiple blocks at once
  BlockDeviceError read (BlockNo block, void* addr, BlockCount count);
  BlockDeviceError write (BlockNo block, void* addr, BlockCount count);
};

#endif /* __Unix_Device_h__ */
