#ifndef __Unix_Device_Async_h__
#define __Unix_Device_Async_h__

#include "fs/device/UnixDevice.H"
#include "fs/device/block/BlockDeviceAsync.H"
#include "fs/device/block/BlockDeviceAsyncExecutor.H"


class BlockDeviceAsyncHdl; // forward declaration to avoid circular inclusion

// This class implements the standard interface for block oriented devices
// on a UNIX file device. It also implements the necessary locking for
// multithreaded device access. Asynchronous operations are provided by
// creating concurrent posix threads.
class UnixDeviceAsync : public UnixDevice, public virtual BlockDeviceAsync
{
  BlockDeviceAsyncExecutor executor;
public:
  UnixDeviceAsync (const char* name) : UnixDevice (name), executor (this) {}

  // read and write multiple blocks at once
  BlockDeviceError read_async (BlockNo block, void* addr, BlockCount count, BlockDeviceAsyncHdl* hdl)
  {
    return executor.read_async (block, addr, count, hdl);
  }

  BlockDeviceError write_async (BlockNo block, void* addr, BlockCount count, BlockDeviceAsyncHdl* hdl)
  {
    return executor.write_async (block, addr, count, hdl);
  }
};

#endif /* __Unix_Device_Async_h__ */
