#ifndef __Unix_Device_User_h__
#define __Unix_Device_User_h__

#include "fs/device/UnixDeviceError.H"
#include "fs/device/UnixDeviceIdentification.H"

#include <fcntl.h>

// This class implements the device access layer for "unix file devices".
// The access permissions or granted (or not) with a call the 'open' method.
// Clients use 'close' to free the allocated device (i.e. unix file).
// Both methods return 'false' if an error occurrs.
// Extended error information can be retrieved with the 'error' method.
// 'error' is called after the operation that failed. Make sure that in
// case of concurrency no other thread calls another device function
// before the error information is read.
class UnixDeviceUser : public UnixDeviceIdentification
{
  int fd;                    // the unix file descriptor
public:
  UnixDeviceUser (const char* devname) : 
  UnixDeviceIdentification (devname) {}

  enum mode 
  { 
    R  = O_RDONLY, // open mode: read, write or read/write
    W  = O_WRONLY, 
    RW = O_RDWR
  };

  bool open (mode m = RW);   // try to access the device
  bool close ();             // release a device

  int handle ();             // get the device handle to access the device
  UnixDeviceError error ();  // return the error code of the last failed operation
};

#endif /* __Unix_Device_User_h__ */
