#ifndef __Unix_Device_Worker_h__
#define __Unix_Device_Worker_h__

#include "fs/device/UnixDeviceInformer.H"
#include "fs/device/UnixDeviceWorkerError.H"

// This class implements the real working part of the Unix file device
// driver: It allows reading and writing blocks as well as formatting
// the device. Formatting means to fill all blocks with zeros.
class UnixDeviceWorker : public UnixDeviceInformer
{
  UnixDeviceWorkerError::Code error_code;
  bool seek(unsigned long block);
public:
  UnixDeviceWorker (const char* name) : UnixDeviceInformer (name), error_code (UnixDeviceWorkerError::NO_ERROR) {}

  // read, write, and format a single block
  bool read_block (unsigned long block, void* addr);
  bool write_block (unsigned long block, void* addr);
  bool format_block (unsigned long block);

  // read and write multiple blocks at once
  bool read (unsigned long block, void* addr, unsigned long count);
  bool write (unsigned long block, void* addr, unsigned long count);

  // format the whole device; if the size of the device is unknown it
  // can be specified as a parameter
  bool format (unsigned long count = 0);

  UnixDeviceWorkerError error ();
};

#endif /* __Unix_Device_Worker_h__ */
