#ifndef __EXT2File_h__
#define __EXT2File_h__

#include "fs/FSFileImpl.H"
#include "fs/FSObj.H"

#include "fs/ext2/EXT2FS.H"

/**
 * This class implements the interface of FSFileImpl. It implements read and 
 * write acces to a file. Operation on references are forwarded to EXT2Obj.
 */
class EXT2File : public FSFileImpl
{
public:
  /**
   * Default constructor of the EXT2File class.
   *
   * @param filesystem pointer to the filesystem
   * @param inodenr number of the starting inode
   */
  EXT2File(EXT2FS *filesystem, unsigned long inodenr);
  ~EXT2File() {}

  /**
   * This method identifies this object as a file
   *
   * @return Returns the type of the object identifying it as a file.
   */
  FSObj::Type type() { return FSObj::File; }

  /**
   * Reads data from the file to a buffer.
   *
   * @param buffer buffer to read to
   * @param buffer_size size of the buffer
   * @param pos start position in the file
   * @param really_transfered number of bytes read 
   * @return Returns the error status of the filesystem as an FSResult instanze 
   *         (default is FSResult::NoError).
   */
  FSResult read(void* buffer, FS::Count buffer_size, FS::Pos pos, FS::Count &really_transfered);

  /**
   * Writes a number of bytes from a buffer to the file.
   * 
   * @param buffer contains the data that will be written
   * @param bytes_to_write number of bytes to write
   * @param pos start position in the file
   * @return Returns the error status of the filesystem as an FSResult instanze 
   *         (default is FSResult::NoError).
   */
  FSResult write(void* buffer, FS::Count bytes_to_write, FS::Pos pos); 

  /**
   * Truncates the file to the given length.
   *
   * @param pos new max. lenght of the file (if pos is bigger than the actual 
   *        size of the file nothin is done.
   * @return Returns the error status of the filesystem as an FSResult instanze 
   *         (default is FSResult::NoError).
   */
  FSResult trunc(FS::Pos pos);
};

#endif /* __EXT2File_h__ */
