#ifndef __Async_Operation_Hdl_h__
#define __Async_Operation_Hdl_h__

// This class provides an interface for asynchronous operation handles
// which are used as a per-operation context for asynchronous activities
// and as a handle for synchronization of the initiating thread and the
// asynchronous activity.
// The class also implements the necessary posix thread operations for
// asynchronous execution and synchronization.

class AsyncOperationHdl
{
private:
  // entry point for the posix thread. "arg" is the OperationHdl itself.
  static void* thread_entry (void* arg);

protected:

  // this routine is called by "thread_entry" and must be specialized
  // by derived classes. It is not allowed to call "action" directly,
  // because it runs as a thread.
  virtual void action () = 0;

  // this routine is called automatically by "thread_entry" when the
  // "action" is done.
  virtual void done () {}
public:
  // start the asynchronous activity.
  bool execute ();
  virtual ~AsyncOperationHdl () {}
};

#endif /* __Async_Operation_Hdl_h__ */
