#include "fs/FS.H"
#include "fs/FSDir.H"
#include "fs/FSDirImpl.H"
#include "fs/FSFile.H"
#include "fs/FSFileImpl.H"

#include <iostream>

FSDir::FSDir () : FSObj () {} 

FSDir::FSDir (FS* filesystem)
{
  init (0, 0, filesystem->root ());
  impl_dir ()->new_reference ();
}
   
FSResult FSDir::read_entry (char* buffer, int &buffer_size, FSObj::Type &type)
{
  FSResult result;

  if ((result = check_impl ())) 
  {
    result = impl_dir ()->read_entry (buffer, buffer_size, type);
  }
  return result;
}

FSResult FSDir::reset ()
{
  FSResult result;

  if ((result = check_impl ()))
    result = impl_dir ()->reset ();

  return result;
}

FSResult FSDir::get_entry (const char* name, FSDir &dir)
{
  FSResult result;
  FSDirImpl* dir_obj;

  if ((result = check_impl ()))
  {
    if ((result = impl_dir ()->get_entry (name, dir_obj)))
    {
      dir.release ();
      dir.init (name, impl_dir (), dir_obj);
    }
  }
  return result;
}

FSResult FSDir::get_entry (const char* name, FSFile &file)
{
  FSResult result;
  FSFileImpl* file_obj;

  if ((result = check_impl ()))
  {
    if ((result = impl_dir ()->get_entry (name, file_obj)))
    {
      file.release ();
      file.init (name, impl_dir (), file_obj);
    }
  }
  return result;
}

FSResult FSDir::create (const char* name, FSFile &file)
{
  FSResult result;
  FSFileImpl* file_obj;

  if ((result = check_impl ()))
  {
    if ((result = impl_dir ()->create(name, file_obj)))
    {
      file.release ();
      file.init (name, impl_dir (), file_obj);
    }
  }
   return result;
}

FSResult FSDir::mkdir (const char* name, FSDir &dir)
{
  FSResult result;
  FSDirImpl* dir_obj;

  if ((result = check_impl ()))
  {
    if ((result = impl_dir ()->mkdir(name, dir_obj)))
    {
      dir.release ();
      dir.init (name, impl_dir (), dir_obj);
    }
  }
  return result;
}
