#include "fs/FSObj.H"

#include "fs/FSObjImpl.H"
#include "fs/FSDirImpl.H"

#include <iostream>
#include <string.h>

FSResult FSObj::check_impl ()
{
  return (impl ? FSResult::NoError : FSResult::UninitializedObj);
}

FSResult FSObj::check_dir ()
{
  return (dir ? FSResult::NoError : FSResult::InvalidForRoot);
}

void FSObj::release ()
{
  if (impl)
  {
    if (dir) dir->release_reference ();
    impl->release_reference ();
  }
  if (itsmybuffer) 
  {
    delete[]name;
    name = NULL;
  }
} 

void FSObj::init (const char* obj_name, FSDirImpl *directory, FSObjImpl *object)
{
  name = obj_name;

  if (obj_name) 
  {
    name = new char [strlen(obj_name)+1];
    if (!name) 
    {
      name = obj_name;
      itsmybuffer = 0;
    } else {
      strcpy((char*)name, obj_name);
      itsmybuffer = 0;
    }
  }

  dir  = directory;
  impl = object;

  if (impl)
  {
    if (dir) dir->new_reference ();
    impl->new_reference ();
  }
}

void FSObj::init (const FSObj& copy)
{
  init (copy.name, copy.dir, copy.impl);
}

FSResult FSObj::move (FSDir& location, const char* new_name = 0)
{
  FSResult result;

  if ((result = check_impl ()) && (result = check_dir ()))
    result = dir->move (name, (FSDirImpl*)((FSObj&)location).impl, new_name);
  return result;
}

FSResult FSObj::remove ()
{
  FSResult result;

  if ((result = check_impl ()) && (result = check_dir ()))
    result = dir->remove (name);  
  return result;
}

FSResult FSObj::link (FSDir& location, const char* link_name = 0)
{
  FSResult result;

  if ((result = check_impl ()) && (result = check_dir ()))
    result = dir->link (name, (FSDirImpl*)((FSObj&)location).impl, link_name);
  return result;
}
      
FSResult FSObj::attribute_count (int &count)
{
  FSResult result;

  if ((result = check_impl ()) && (result = check_dir ()))
    result = dir->attribute_count (name, count);
  return result;
}

FSResult FSObj::attribute_name (int n, const char *&attr_name)
{
  FSResult result;

  if ((result = check_impl ()) && (result = check_dir ()))
    result = dir->attribute_name (name, n, attr_name);
  return result;
}

FSResult FSObj::attribute_description (int n, const char *&description)
{
  FSResult result;

  if ((result = check_impl ()) && (result = check_dir ()))
    result = dir->attribute_description (name, n, description);
  return result;
}

FSResult FSObj::set_attribute (const char* attr_name, const char *val)
{
  FSResult result;

  if ((result = check_impl ()) && (result = check_dir ()))
    result = dir->set_attribute (name, attr_name, val);
  return result;
}

FSResult FSObj::get_attribute (const char* attr_name, char *val, int& size)
{
  FSResult result;

  if ((result = check_impl ()) && (result = check_dir ())) {
    result = dir->get_attribute (name, attr_name, val, size);
  }
  return result;
}
