#include "fs/device/UnixDevice.H"

static const char* err_mutex = "problem with mutex variable";

// constructor
UnixDevice::UnixDevice (const char* name) : UnixDeviceLogic (name)
 {
   if (!UnixDeviceLogic::open (UnixDeviceLogic::RW))
    {
      if (!UnixDeviceLogic::open (UnixDeviceLogic::R))
	 access = NONE;
      else
         access = R;
    }
   else
      access = RW;

   if (access != NONE)
      detect ();
 }


// destructor
UnixDevice::~UnixDevice ()
 {
   UnixDeviceLogic::close ();
 }


// select the preferred logical block size
BlockDeviceError UnixDevice::setup (BlockSize block_size)
 {
   const char* msg = (const char*)0;

   if (!mutex.lock ())
      return BlockDeviceError (err_mutex); 

   if (access == RW)
      UnixDeviceLogic::setup (block_size);
   else
      msg = "setting block size is not possible, you have no RW permission"; 

   if (!mutex.unlock ())
      return BlockDeviceError (err_mutex); 

   return BlockDeviceError (msg);
 }

BlockDevice::BlockSize UnixDevice::block_size ()
 {
   return (BlockSize)phys_block_size ();
 }


BlockDevice::BlockCount UnixDevice::block_count ()
 {
   return (BlockCount)phys_blocks ();
 }


// read and write multiple blocks at once

BlockDeviceError UnixDevice::read (BlockNo block, void* addr, 
				   BlockCount count)
 {
   const char* msg = (const char*)0;

   if (!mutex.lock ())
      return BlockDeviceError (err_mutex); 

   if (!UnixDeviceLogic::read (block, addr, count))
      msg = error ();

   if (!mutex.unlock ())
      return BlockDeviceError (err_mutex); 

   return BlockDeviceError (msg);
 }


BlockDeviceError UnixDevice::write (BlockNo block, void* addr, 
				    BlockCount count)
 {
   const char* msg = (const char*)0;

   if (!mutex.lock ())
      return BlockDeviceError (err_mutex); 

   if (!UnixDeviceLogic::write (block, addr, count))
         msg = error ();

   if (!mutex.unlock ())
      return BlockDeviceError (err_mutex); 

   return BlockDeviceError (msg);
 }
