#include "fs/device/UnixDeviceWorker.H"

#include <sys/types.h>
#include <unistd.h>


// set the file pointer to the requested block
bool UnixDeviceWorker::seek (unsigned long block)
{
  if (lseek (handle (), (off_t)(block * phys_block_size ()), SEEK_SET) == (off_t)-1)
    return false;
  else
    return true;
}

// read, write, and format a single block
bool UnixDeviceWorker::read_block (unsigned long block, void* addr)
{
  if (!seek (block))
    return false;
  if (((unsigned long)::read (handle (), addr, phys_block_size ())) != phys_block_size ())
  {
    error_code = UnixDeviceWorkerError::END_OF_FILE;
    return false;
  }
  return true;
}
   
bool UnixDeviceWorker::write_block (unsigned long block, void* addr)
{
  if (!seek (block))
    return false;
  return (((unsigned long)::write (handle (), addr, phys_block_size ())) == phys_block_size ());
}

bool UnixDeviceWorker::format_block (unsigned long block)
{
  char zero = (char)0;

  if (!seek (block))
    return false;

  for (unsigned long pos = 0; pos < phys_block_size (); pos++)
    if (::write (handle (), &zero, 1) != 1)
      return false;
  return true;
}

// read and write multiple blocks at once
bool UnixDeviceWorker::read (unsigned long block, void* addr, unsigned long count)
{
  if (!seek (block))
    return false;
  unsigned long total = count * phys_block_size ();

  if (((unsigned long)::read (handle (), addr, total)) != total)
  {
    error_code = UnixDeviceWorkerError::END_OF_FILE;
    return false;
  }
  return true;
}
   
bool UnixDeviceWorker::write (unsigned long block, void* addr, unsigned long count)
{
  if (!seek (block))
    return false;
  unsigned long total = count * phys_block_size ();
  return (((unsigned long)::write (handle (), addr, total)) == total);
}


// format the whole device; if the size of the device is unknown it
// can be specified as a parameter
bool UnixDeviceWorker::format (unsigned long count)
{
  if (!seek (0L))
    return false;
  if (!phys_blocks () && count)
    setup (count, phys_block_size ());
  for (unsigned long block = 0; block < phys_blocks (); block++)
    if (!format_block (block))
      return false;
  return true;
}

UnixDeviceWorkerError UnixDeviceWorker::error ()
{
  UnixDeviceWorkerError::Code code;
  if (error_code == UnixDeviceWorkerError::NO_ERROR)
    return UnixDeviceWorkerError (UnixDeviceInformer::error ());
  code = error_code;
  error_code = UnixDeviceWorkerError::NO_ERROR;
  return UnixDeviceWorkerError (code);
}
