#include "fs/ext2/EXT2Dir.H"
#include "fs/ext2/EXT2File.H"
#include "fs/ext2/attributs.h"
#include "fs/ext2/errno.h"

#include <iostream>
#include <stdio.h> 		// for sprintf
#include <string.h>		// for strdup

//------------------------------------------------------------------------------
extern "C"
{
  FSResult str2perm(const char *val, unsigned short &perm);
  FSResult perm2str(unsigned short perm, char *val, int &size);
  FSResult type2str(unsigned short mode, char *&val, int &size);
}
      

static struct attrib attribs[] = 
{
  { "size", "size of the filesystem object" } ,
  { "uid", "user-id of the owner" } ,
  { "gid", "group-id of the owner" } ,
  { "type", "inode type: chardev, blockdev, fifo, dir, file, symlink, socket, unknown" } ,
  { "perm", "file permissions: owner/group/others: rwxr-xr-x + setuid + sticky" } ,
  { "atime", "access time" } ,
  { "ctime", "allocation time" } ,
  { "mtime", "modification time" } ,
  { "inode", "inode number" },
  { "links_count", "number of links to this object" },
  { "blocks", "number of used blocks" }
};

EXT2Dir::EXT2Dir(EXT2FS *filesystem, unsigned long inode) : FSDirImpl(filesystem) 
{
  //TODO Inode anlegen
}

FSResult EXT2Dir::create(const char* name, FSFileImpl *&file) 
{
  int result = EXT2E_NOERROR;
 
  //TODO - find free inode
  //     - initialise inode
  //     - create EXT2File
  //     - insert directroy entry
  switch (result) {
    case EXT2E_NOERROR : return FSResult::NoError;
    case EXT2E_DIR_NOSUCHFILE : return FSResult::FileNotFound;
    case EXT2E_DIR_NOMEM : return FSResult::UninitializedObj;
    default : return FSResult::FsSpecificError;
  }
}

FSResult EXT2Dir::get_entry(const char* name, FSDirImpl *&dir)
{
  int result = 0;

  //TODO - find name in directory -> get inode from entry
  //     - check if entry is a directory
  //     - create EXT2Dir with found entry
  if (!dir) {
    switch (result) {
      case EXT2E_DIR_NOSUCHFILE : return FSResult::FileNotFound;
      case EXT2E_DIR_NOMEM : return FSResult::UninitializedObj;
      default : return FSResult::FsSpecificError;
    }
  }
  return FSResult::NoError;
}

FSResult EXT2Dir::get_entry(const char* name, FSFileImpl *&file)
{
  int result = 0;

  //TODO - find name in directory -> get inode from entry
  //     - check if entry is a file
  //     - create EXT2Dir with found entry
  if (!file) {
    switch (result) {
      case EXT2E_DIR_NOSUCHFILE : return FSResult::FileNotFound;
      case EXT2E_DIR_NOMEM : return FSResult::UninitializedObj;
      default : return FSResult::FsSpecificError;
    }
  }
  return FSResult::NoError;
}

FSResult EXT2Dir::mkdir(const char* name, FSDirImpl *&dir) 
{ 
  int result;

  //TODO - find free inode
  //     - initialise inode
  //     - create EXT2Dir
  //     - initialise directory structure -> '.', '..'
  //     - insert directroy entry
  switch (result) {
    case EXT2E_NOERROR : return FSResult::NoError;
    case EXT2E_DIR_NOSUCHFILE : return FSResult::FileNotFound;
    case EXT2E_DIR_NOMEM : return FSResult::UninitializedObj;
    default : return FSResult::FsSpecificError;
  }
}

FSResult EXT2Dir::move(const char *name, FSDirImpl *location, const char* new_name)
{
  int result;

  //TODO - find name in directory -> get entry
  //     - insert entry with new name in location
  //     - remove entry from current directory
  switch (result) {
    case EXT2E_NOERROR : return FSResult::NoError;
    case EXT2E_DIR_NOSUCHFILE : return FSResult::FileNotFound;
    case EXT2E_DIR_NOMEM : return FSResult::UninitializedObj;
    case EXT2E_DIR_OWN_SUBSTRUCTURE : return FSResult::OwnSubstructure;
    default : return FSResult::FsSpecificError;
  }
}

FSResult EXT2Dir::link(const char *name, FSDirImpl *destination, const char* new_name) 
{
  int result;

  //TODO - find name in directory -> get inode from entry
  //     - insert new entry into destination (with new name)
  switch (result) {
    case EXT2E_NOERROR : return FSResult::NoError;
    case EXT2E_DIR_NOSUCHFILE : return FSResult::FileNotFound;
    case EXT2E_DIR_NOMEM : return FSResult::UninitializedObj;
    default : return FSResult::FsSpecificError;
  }
}

FSResult EXT2Dir::read_entry(char* buffer, int &buffer_size, FSObj::Type &type)
{
  int result = 0;
  unsigned long mode;

  //TODO insert source code
  switch (result) {
    case EXT2E_DIR_BUFFER_TOO_SMALL : return FSResult::BufferTooSmall;
    case EXT2E_DIR_NOSUCHFILE : return FSResult::FileNotFound;
    case EXT2E_DIR_EOF : return FSResult::EndOfFile;
    default : return FSResult::FsSpecificError;
  }
  type = (mode & 0040000) ? FSObj::Directory :
         (mode & 0100000) ? FSObj::File : FSObj::Unknown;
  return FSResult::NoError;
}

FSResult EXT2Dir::remove(const char *name) 
{
  int result;

  //TODO - find name in directory -> get entry
  //     - remove entry from directory
  //     - free occupied inodes and datablocks
  switch (result) {
   case EXT2E_NOERROR: return FSResult::NoError;
   case EXT2E_DIR_NOSUCHFILE: return FSResult::FileNotFound;
   case EXT2E_DIR_NOTEMPTY: return FSResult::NotEmpty;
   default: return FSResult::FsSpecificError;
  }
}

FSResult EXT2Dir::reset()
{
  //TODO insert source code
  return FSResult::NoError;
}

FSResult EXT2Dir::attribute_count (const char* name, int &count) 
{
  count = NUMBER_OF_ATTRIBS;
  return FSResult::NoError;
}

FSResult EXT2Dir::attribute_name (const char* name, unsigned int n, const char *&attr_name) 
{
  if (n > NUMBER_OF_ATTRIBS) return FSResult::AttribUnknown;
  attr_name = attribs[n].name;
  return FSResult::NoError;
}

FSResult EXT2Dir::attribute_description (const char *name, const unsigned int n, const char *&description) 
{
  if (n > NUMBER_OF_ATTRIBS) return FSResult::AttribUnknown;
  description = attribs[n].desc;
  return FSResult::NoError;
}

FSResult EXT2Dir::get_attribute(const char *name, const char* attr_name, char *val, int& size)  
{
  //TODO - get attribut data from object <name>
  for (int i = 0; i < 5;i++)
  {
    if (strcmp(attr_name,attribs[i].name)==0)
    {
      switch (i)
      {
        case 0: // size
          //TODO insert source code
          break;
        case 1: // uid
          //TODO insert source code
          break;
        case 2: // gid
          //TODO insert source code
          break;
        case 3: // type
          //TODO insert source code
          break;
        case 4: // perm
          //TODO insert source code
          break;
        case 5: // atime
          //TODO insert source code
          break;
        case 6: // ctime
          //TODO insert source code
          break;
        case 7: // mtime
          //TODO insert source code
          break;
        case 8: // inode
          //TODO insert source code
          break;
        case 9: // links_count
          //TODO insert source code
          break;
        case 10: // blocks
          //TODO insert source code
          break;
      }
      if (size<0) return FSResult::BufferTooSmall;
      else return FSResult::NoError;
    }
  }
  return FSResult::AttribUnknown;
}

FSResult EXT2Dir::set_attribute(const char *name, const char* attr_name, const char *val)  
{
  //TODO - get attribut data from object <name>
  for (int i = 0; i < 5;i++)
  {
    if (strcmp(attr_name,attribs[i].name)==0)
    {
      switch (i)
      {
        case 0: // size
          //TODO insert source code
          break;
        case 1: // uid
          //TODO insert source code
          break;
        case 2: // gid
          //TODO insert source code
          break;
        case 3: // type
          //TODO insert source code
          break;
        case 4: // perm
          //TODO insert source code
          break;
        case 5: // atime
          //TODO insert source code
          break;
        case 6: // ctime
          //TODO insert source code
          break;
        case 7: // mtime
          //TODO insert source code
          break;
        case 8: // inode
          //TODO insert source code
          break;
        case 9: // links_count
          //TODO insert source code
          break;
        case 10: // blocks
          //TODO insert source code
          break;
      }
      return FSResult::NoError;
    }
  }
  return FSResult::AttribUnknown;
}
