#include "fs/ext2/EXT2FS.H"
#include "fs/ext2/EXT2Dir.H"
#include "fs/ext2/EXT2File.H"
#include "fs/ext2/attributs.h"
#include "fs/ext2/errno.h"

#include <iostream>

//------------------------------------------------------------------------------
extern "C"
{
  FSResult str2perm(const char *val, unsigned short &perm);
  FSResult perm2str(unsigned short perm, char *val, int &size);
  FSResult type2str(unsigned short mode, char *&val, int &size);
}

static struct attrib attribs[] = 
{
  { "readonly"  , "Is the readonly mode set ? (\"yes\"/\"no\")"},
  { "uid"     , "UID for file access (number)" }, 
  { "gid"     , "GID for for file access (number)" }, 
  { "umask"   , "Unix style UMASK for file access (number)" }, 
  { "type"    , "filetype for creating new files (\"symlink\"/\"file\")" } 
};

EXT2FS::EXT2FS(BlockDevice *dev) : device(dev) 
{
  errno = 0;
  root_dir = NULL;

  if (!dev) {
    errno = EXT2E_FS_NULL_DEVICE;
    return;
  }
  
  //TODO create/read inode table & open root directory -> inode number of root is 2
  if (!root_dir) {
    errno = EXT2E_FS_ROOT_FAILED;
    return;
  }
}

EXT2FS::~EXT2FS()
{
  //TODO delete root directory
}


FSResult EXT2FS::check()
{
  switch (errno) 
  {
    case 0:                                                 // everything's fine
      return FSResult::NoError;
    case EXT2E_BLKDEV_INIT_FAILED|EXT2E_BLKDEV_READ_FAILED: // BlockDevice failed
      return FSResult::FsSpecificError;
    case EXT2E_FS_NULL_DEVICE:                              // NULL pointer as device or no memory for itable
      return FSResult::UninitializedObj;
    case EXT2E_FS_INIT_FAILED|EXT2E_FS_ROOT_FAILED:
      return FSResult::UninitializedObj;
    case EXT2E_SUPERBLK_BADMAGICNUMBER:
      return FSResult::InvalidFS;
    default:                                                // hmm... dunno
      return FSResult::FsSpecificError;
  }
}

FSResult EXT2FS::sync()
{
  if (errno) return check();
  //TODO synchronise inode table
  int result = EXT2E_NOERROR;
  switch (result)
  {
    case EXT2E_NOERROR:
      return FSResult::NoError;
    case EXT2E_READONLY:
      return FSResult::Readonly;     
    default: 
      return FSResult::FsSpecificError;
  }
}

FSDirImpl* EXT2FS::root()
{
  return root_dir;
}

FSResult EXT2FS::attribute_count(int &count)
{
  count = NUMBER_OF_ATTRIBS;
  return FSResult::NoError;
}

FSResult EXT2FS::attribute_name(unsigned int n, const char *&name)
{
  if (n > NUMBER_OF_ATTRIBS) return FSResult::AttribUnknown;
  name = attribs[n].name;
  return FSResult::NoError;
}

FSResult EXT2FS::attribute_description(unsigned int n, const char *&description)
{
  if (n > NUMBER_OF_ATTRIBS) return FSResult::AttribUnknown;
  description = attribs[n].desc;
  return FSResult::NoError;
}

FSResult EXT2FS::get_attribute (const char* name, char *val, int& size)
{ 
  for (int i = 0; i < 5;i++)
  {
    if (strcmp(name,attribs[i].name)==0)
    {
      switch (i) 
      {
        case 0: // readonly
          //TODO insert source code
          break; 
        case 1: // uid
          //TODO insert source code
          break; 
        case 2: // gid
          //TODO insert source code
          break; 
        case 3: // umask
          //TODO insert source code
          break; 
        case 4: // type
          //TODO insert source code
          break;
      }
      if (size<0) return FSResult::BufferTooSmall;
      else return FSResult::NoError; 
    }
  }
  return FSResult::AttribUnknown;
}

FSResult EXT2FS::set_attribute(const char* name, const char *val)
{
  for (int i = 0; i < 5;i++)
  {
    if (strcmp(name,attribs[i].name)==0)
    {
      switch (i) 
      {
        case 0: // readonly
          //TODO insert source code
          break; 
        case 1: // uid
          //TODO insert source code
          break; 
        case 2: // gid
          //TODO insert source code
          break; 
        case 3: // umask
          //TODO insert source code
          break; 
        case 4: // type
          //TODO insert source code
          break;
      }
      return FSResult::NoError;
    }
  }
  return FSResult::AttribUnknown;
}
