#define BUFFER_SIZE 512

#include "UnixDevice.H"
#include "EXT2FS.H"
#include "FSDir.H"
#include "FSFile.H"

#include <iostream>
#include <string.h>
#include <ctype.h>
#include <limits.h>
#include <stdio.h>

// GLOBAL VARIABLES
char *thisisbuffer = NULL;
char buffer[BUFFER_SIZE];
char cwd_path[1024];            // String of current working directory
const char delimiter[]  = "/";  // Filename delimiter in pathnames
const char parent_dir[] = ".."; // Filename for parent dir

FS *fs;                         // Filesystem
FSDir root;                     // Root directory
FSDir cwd;                      // Current working directory

// METHODS

// -----------------------------------------------------------------------------
// support methods
void print_help() {
  printf("Available commands:\n");
  printf("help\n");
  printf("ls          [path]\n");
  printf("cd          [path]\n");
  printf("cp          <source> <destination>\n");
  printf("cat         <file>\n");
  printf("attribs     <name>\n");
  printf("setattrib   <name> <attrname> <attrvalue>\n");
  printf("fsattribs\n");
  printf("setfsattrib <attrname> <attrvalue>\n");
  printf("mv          <source> <destination>\n");
  printf("rm          <name>\n");
  printf("ln          <target> <destination>\n");
  printf("create      <name>\n");
  printf("mkdir       <name>\n");
  printf("diff        <file> <file>\n");
  printf("sync\n");
  printf("exit\n\n");
}

bool find_dir(FSDir root, const char *path, FSDir &dir, char *new_path, bool parent = false)
{
  char buffer[1024];
  char *ptr = buffer;
  char *part;
  FSDir prev, next;
  FSResult result;
  bool have_root;

  // order of strcpys is important, if called with path == new_path!
  strcpy (buffer, path);
  strcpy (new_path, cwd_path);
   
  while ( (part = strsep(&ptr, delimiter)) )
  {
    if (strcmp (part, "") == 0)
    {
      have_root = true;
      dir = root;
      strcpy (new_path, "/");
    } else if (strcmp (part, parent_dir) == 0)
    {
      if (strcmp (new_path, delimiter) == 0)
      {
        printf("Can't get parent of root directory\n");
        return false;
      }
      FSDir dummy;
      if (!find_dir(root, new_path, dummy, new_path, true)) return false;
      dir = dummy;
    } 
    else
    {
      have_root = false;
      result = dir.get_entry(part, next);
      if (result) 
      {
      	prev = dir;
      	dir = next;
      	if (new_path[strlen(new_path) - 1] != delimiter[0])
      	  strcat(new_path, delimiter);
      	strcat(new_path, part);
      }
      else
      {
        printf("%s\n", result.message());
      	return false;
      }
    }
  }
   
  if (parent) 
  {
    if (have_root)
    {
      printf("Can't get parent of root directory\n");
      return false;
    }
    else
    {
      char* ptr = &new_path[strlen (new_path) - 1];
      while (*ptr != delimiter[0]) ptr--;
      if (ptr != new_path)
      	*ptr = '\0';
      else
       	strcpy(new_path , delimiter);
      dir = prev;
    }
  }
  return true;
}

void list(FSDir dir)
{
  int len;
  char buffer[256];
  FSObj::Type type;
  FSResult result;

  do 
  {
    len = sizeof(buffer);
    result = dir.read_entry(buffer, len, type);
    if (!result) break;

    printf("%-20s ",buffer);
    if (type == FSObj::Directory) printf("<directory>");
    else if (type == FSObj::File) printf("<file>");
    else if (type == FSObj::Unknown) printf("<unknown>");
    printf("\n");
  } while (true);
   
  if (result != FSResult (FSResult::EndOfFile))
    printf("%s\n", result.message());
  dir.reset ();
}

FSObj *make_obj(FSDir root, const char *path)
{
  char buffer[1024];
  char new_path[1024];

  if (strcmp (path, delimiter) == 0)
    return new FSDir (root);

  if (path[strlen (path) - 1] == delimiter[0])
  {
    printf("invalid argument\n");
    return 0;
  }

  strcpy (buffer, path);

  FSDir dirobj;
  FSFile fileobj;
  FSDir dir;
  FSResult result;
  char* ptr = &buffer [strlen (buffer) - 1];

  thisisbuffer = buffer;
  while ((ptr != buffer) && (*ptr != delimiter[0])) ptr--;
   
  if ((ptr==buffer) && (*ptr==delimiter[0])) { dir = root; ptr++; } else dir=cwd;
  if (*ptr == delimiter[0])
  {
    *ptr = '\0';
    ptr++;
    if (!find_dir(root, buffer, dir, new_path)) return 0;
  }

  if ((result = dir.get_entry(ptr, dirobj)))
    return new FSDir(dirobj);
  else if (result == FSResult::NoDirectory)
  {
    if ((result = dir.get_entry(ptr, fileobj)))
      return new FSFile(fileobj);
  }
  printf("%s\n", result.message());
  return 0;
}

void pathtok(char *input,char *&path,char *&file)
{
  char h[1024]; 
  int l=strlen(delimiter); 
 
  if ( (!input) || (strcmp(input,delimiter) == 0) ) 
  {
    path=NULL; 
    file=NULL;
    return;
  }
  
  if (strncmp(input, delimiter,l) != 0)
  {
    strcpy(h, cwd_path); 
    if (strcmp(h, delimiter) != 0) strcat(h, delimiter);
    strcat(h, input);
    strcpy(input, h);
  }
  file = input + strlen(input);
  while( strncmp(file, delimiter,l) != 0 ) 
  {
    (file)--;
  }
  (file)[0] = '\0';
  (file) += l; 
  path =input;
  if ((file)[0] == '\0') file = NULL;
  if ((path)[0] == '\0') path = NULL;
  return;
}

// -----------------------------------------------------------------------------
// main commands
void cmd_attribs (int argc, char** argv)
{
  FSResult result;
  FSObj *obj;

  if ( argc != 2)
  {
    printf("parameter is missing\n");
    return;
  }

  if ( !(obj = make_obj(root, argv[1])) ) return;
  int count;
  if ( !(result = obj->attribute_count(count)) )
  {
    printf("%s\n", result.message());
    delete obj;
    return;
  }

  for (int a = 0; a < count; a++)
  {
    const char *name;
    char val[100];
    int size = sizeof (val);

    if ( (result = obj->attribute_name(a, name)) )
    {
      if ( (result = obj->get_attribute(name, val, size)) )
      {
        printf("%s\t%s\n", name, val);
      }
    }
    if (!result)
    {
      printf("%s\n", result.message());
      delete obj;
      return;
    }
  }
  delete obj;
}

void cmd_setattrib(int argc, char** argv)
{
  FSResult result;
  FSObj *obj;

  if (argc !=  4)
  {
    printf("parameter is missing\n");
    return;
  }
  if ( !(obj = make_obj(root, argv[1])) ) return;
  if ( !(result=obj->set_attribute(argv[2],argv[3])) )
  {
    printf("%s\n", result.message());
  } 
  delete obj;
  return;
}

void cmd_fsattribs (int argc, char** argv)
{
  FSResult result;

  if (argc != 1)
  {
    printf("invalid parameters\n");
    return;
  }

  int count;
  if ( !(result = fs->attribute_count(count)) )
  {
    printf("%s\n", result.message());
    return;
  }

  for (int a = 0; a < count; a++)
  {
    const char *name;
    char val[100];
    int size = sizeof(val);

    if ((result = fs->attribute_name(a, name)) && (result = fs->get_attribute(name, val, size)))
    {
      printf("%s\t%s\n", name, val);
    }
    else
    {
      printf("%s\n", result.message());
      return;
    }
  }
}

void cmd_setfsattrib (int argc, char** argv)
{
  FSResult result;

  if ( argc != 3 )
  {
    printf("invalid parameters\n");
    return;
  }
  
  if (!(result = fs->set_attribute(argv[1],argv[2])))
  {
    printf("%s\n", result.message());
    return;
  }
}

void cmd_cat (int argc, char** argv)
{
  FSObj *obj;

  if (argc != 2)
  {
    printf("invalid arguments\n");
    return;
  }

  if ( !(obj = make_obj(root, argv[1])) ) return;
  if ( obj->type() != FSObj::File )
  {
    printf("object is not a regular file\n");
    delete obj;
    return;
  }

  char copy_buffer[4097];
  FS::Pos pos = 0;
  FS::Count trans;
  FSResult result;
  while ( result = ((FSFile*)obj)->read(copy_buffer, sizeof (copy_buffer) - 1, pos, trans) )
  {
    pos += trans;
    copy_buffer[trans] = '\0';
    printf("%s", copy_buffer);
  }

  if (result != FSResult::EndOfFile)
  {
    printf("%s\n", result.message());
  }
  delete obj;
}

void cmd_cd(int argc, char** argv)
{
  char new_path[1024];

  if (argc != 2)
  {
    cwd = root;
    strcpy(cwd_path, delimiter);
  }
  else
  {
    FSDir dir = cwd;
    if ( find_dir(root, argv[1], dir, new_path))
    {
      cwd = dir;
      strcpy(cwd_path, new_path);
    }
  }
}

void cmd_copy(int argc, char** argv)
{
   char dest[1024];
   char new_path[1024];
   char *path,*file;
   unsigned char copy_buffer[1024];

  FS::Pos pos = 0;
  FS::Count trans;
  FSDir dir;
  FSObj *obj;
  FSFile fileobj;
  FSResult result;

  if (argc != 3)
  {
    printf("copy : invalid parameters\n");
    return;
  }
  if ( !(obj = make_obj(root, argv[1])) ) return;
  if ( obj->type() != FSObj::File)
  {
    printf("object is not a regular file\n");
    delete obj;
    return;
  }  
   
  pathtok(argv[2],path,file);
  
  if (path) strcpy(dest,path); else strcpy(dest,delimiter);
  if (!file) pathtok(argv[1],path,file);   
  if (!file)
  {
    printf("can't specify destination file\n");
    delete obj;
    return;
  }
     
  dir = cwd;
  if ( !find_dir(root, dest, dir, new_path) ) 
  {
    delete obj;
    return;
  }
   
  if ( !(result = dir.create(file,fileobj)) )
  {
    printf("%s\n", result.message());
    delete obj;
    return;
  }

  while ((result = ((FSFile*)obj)->read(copy_buffer, 1024, pos, trans)) == FSResult::NoError)     
  {
    if ((result = fileobj.write(copy_buffer, trans, pos)) != FSResult::NoError)
    {
      printf("%s\n", result.message());
      delete obj;
      return;
    }
    pos += trans; 
  }

  if ((result != FSResult::EndOfFile) && (result != FSResult::NoError))
  {
    printf("%s\n", result.message());
  }
  delete obj;
}

void cmd_create(int argc, char** argv)
{
  char new_path[1024];
  char *path,*file;

  if (argc != 2) {
    printf("invalid parameter\n");
    return;
  }

  pathtok(argv[1], path, file); if (!path) path = (char *)delimiter;
  if ( (!file) || (!path) )
  {
    printf("invalid parameter\n");
    return;
  }

  FSDir dir;
  FSResult result;
   
  if (strlen(path))
  { 
    if ( !find_dir(root, path, dir, new_path) ) 
    {
      return;
    }
  } else dir = root;
  FSFile fileobj;
  if (!(result = dir.create (file,fileobj)))
  {
    printf("%s\n", result.message());
  }
  return;
}

void cmd_diff(int argc, char** argv)
{
  FSObj *obj1,*obj2;
  char buf1,buf2;
  FS::Pos pos = 0;
  FS::Count trans;
  FSResult res1,res2;
  
  if (argc != 3)
  {
    printf("invalid parameters\n");
    return;
  }
  if ( !(obj1 = make_obj(root, argv[1])) ) return;
  if (obj1->type() != FSObj::File)
  {
    printf("object is not a regular file\n");
    delete obj1;
    return;
  }
  if ( !(obj2 = make_obj(root, argv[2])) )
  {
    delete obj1;
    return;
  }     
  if ( obj2->type() != FSObj::File )
  {
    printf("object is not a regular file\n");
    delete obj1;
    delete obj2;
    return;
  }
  
  do
  {    
    res1 = (((FSFile*)obj1)->read(&buf1,1, pos, trans));
    res2 = (((FSFile*)obj2)->read(&buf2,1, pos, trans));
    pos++;
  } while ((buf1 == buf2) && (res1 == FSResult::NoError) && (res2 == FSResult::NoError));     
  
  if ( (res1 != FSResult::EndOfFile) && (res1 != FSResult::NoError) )
  {
    printf("%s\n", res1.message());
  }
  else if ((res2 != FSResult::EndOfFile) && (res2 != FSResult::NoError))
  {
    printf("%s\n", res2.message());
  } 
  else if ((res1 != res2) || (buf1 != buf2)) printf("The files differ !!\n");
  else printf("The file contents look equal !\n");
  
  delete obj1;
  delete obj2;
  return;
}

void cmd_exit(int argc, char** argv) 
{
  cwd = root;
  delete fs;
  exit(1); 
} 

void cmd_link(int argc, char** argv)
{
  char dest[1024];
  char new_path[1024];
  char *path,*file;

  if (argc != 3) {
    printf("invalid parameter\n");
    return;
  }
  if (strcmp (argv[1], "") == 0 || strcmp (argv[2], "") == 0 || strcmp (argv[2], delimiter)== 0)
  {
    printf("invalid parameters\n");
    return;
  }
  FSDir dir;
  FSObj *obj;
  FSResult result;
   
  if ( !(obj = make_obj(root, argv[1])) ) return;
  pathtok(argv[2],path,file);
  if (path) strcpy(dest,path); else strcpy(dest,delimiter);
  if (!file) pathtok(argv[1],path,file);   
  if (!file)
  {
    printf("can't specify destination file\n");
    delete obj;
    return;
  }
  if ( !find_dir(root, dest, dir, new_path) ) 
  {
    delete obj;
    return;
  }
  if ( !(result = obj->link(dir,file)) )
  {
    printf("%s\n",result.message());
  }
  delete obj;
  return;
}

void cmd_ls(int argc, char** argv)
{
  char buffer[1024];

  if (argc == 1) {
    list(cwd);
  }
  else
  {
    FSDir dir = cwd;
    if ( find_dir(root, argv[1], dir, buffer))
      list(dir);
  }
}

void cmd_mkdir(int argc, char** argv)
{
  char new_path[1024];
  char *path,*file;

  pathtok(argv[1],path,file); if (!path) path=(char *)delimiter;
   
  if ( (!file) || (!path) )
  {
    printf("invalid parameter\n");
    return;
  }

  FSDir dir;
  FSResult result;
   
  if (strlen(path))
  { 
    if (!find_dir(root, path, dir, new_path)) 
    {
      return;
    }
  } else dir = root;

  FSDir dirobj;
  if (!(result = dir.mkdir(file, dirobj)))
  {
    printf("error: %s\n", result.message());
  }
  return;
}

void cmd_move(int argc, char** argv)
{
   char *path,*file;
   char dest[1024];
   char new_path[1024];

  if (argc != 3)
  {
    printf("invalid parameters\n");
    return;
  }
   
  FSDir dir;
  FSObj *obj;
  FSResult result;
   
  if ( !(obj = make_obj(root, argv[1])) ) return;
  pathtok(argv[2],path,file);
  if (path) strcpy(dest, path); else strcpy(dest, delimiter);
  if (!file) pathtok(argv[1], path, file);   
  if (!file)
  {
    printf("can't specify destination file\n");
    delete obj;
    return;
  }
  dir = cwd;
  if ( !find_dir(root, dest, dir, new_path) ) 
  {
    delete obj;
    return;
  }

  if ( !(result = obj->move(dir,file)) )
  {
    printf("%s\n", result.message());
  }
  delete obj;
}

void cmd_remove(int argc, char** argv)
{

  if (argc != 2 )
  {
    printf("invalid parameter\n");
    return;
  }
   
  FSObj *obj;
  FSResult result;
   
  if ( !(obj = make_obj(root, argv[1])) ) return;
  if ( !(result = obj->remove()) )
  {
    printf("%s\n", result.message());
  }
  delete obj;
}

void cmd_sync(int argc, char** argv)
{
  FSResult result;
 
  if ( (result = fs->sync()) != FSResult::NoError)
  {
    printf("%s\n", result.message());
  }
  return;
}


void execute(int argc, char** argv)
{
  if (strcmp (argv[0], "help") == 0)
    print_help();
  else if (strcmp (argv[0], "ls") == 0)
    cmd_ls (argc, argv);
  else if (strcmp (argv[0], "cd") == 0)
    cmd_cd (argc, argv);
  else if (strcmp (argv[0], "cp") == 0)
    cmd_copy (argc, argv);
  else if (strcmp (argv[0], "cat") == 0)
    cmd_cat (argc, argv);
  else if (strcmp (argv[0], "mv") == 0)
    cmd_move (argc, argv);
  else if (strcmp (argv[0], "rm") == 0)
    cmd_remove (argc, argv);
  else if (strcmp (argv[0], "ln") == 0)
    cmd_link (argc, argv);
  else if (strcmp (argv[0], "create") == 0)
    cmd_create (argc, argv);
  else if (strcmp (argv[0], "mkdir") == 0)
    cmd_mkdir (argc, argv);
  else if (strcmp (argv[0], "attribs") == 0)
    cmd_attribs (argc, argv);
  else if (strcmp (argv[0], "fsattribs") == 0)
    cmd_fsattribs (argc, argv);
  else if (strcmp (argv[0], "setattrib") == 0)
    cmd_setattrib (argc, argv);
  else if (strcmp (argv[0], "setfsattrib") == 0)
    cmd_setfsattrib (argc, argv);
  else if (strcmp (argv[0], "sync") == 0)
    cmd_sync(argc, argv);
  else if (strcmp (argv[0], "diff") == 0)
    cmd_diff(argc, argv); 
  else if (strcmp (argv[0], "exit") == 0)
    printf("exiting !!!\n"); 
  else
	 printf("error: %s unknown command\n", argv[0]);
}

// -----------------------------------------------------------------------------
// main program
int main (int argc, char **argv)
{
  char* root_path;

  if (argc > 1) root_path = argv[1]; 
  else 
  {
    printf("missing argument root_path\nusage: %s root_path\n",argv[0]);
    exit(1);
  }

  UnixDevice dev(root_path);
  fs = new EXT2FS (&dev);
  
  if (fs->check() != FSResult::NoError) 
  {
    FSResult r = fs->check();
    printf("Fehler: %s\n", r.message());
    delete fs;
    exit(0);
  }
  root = FSDir (fs);
  cwd  = root;
  strcpy (cwd_path, delimiter);

  char cmdline[512];
  ssize_t length;

  char *arg_values[10];
  int arg_count = 0;
     
  arg_values[0] = cmdline;  
  do 
  {
    printf("%s> ", cwd_path);
    std::cin.getline(cmdline, 512, '\n');
    length = strlen(cmdline);
    for (int index = 0; index <= length; index++) 
    {
      if ( (cmdline[index] == ' ') || (cmdline[index] == '\t') )
      {
        cmdline[index] = '\0';
        arg_count++;
        arg_values[arg_count] = cmdline + index + 1;
      }
    }
    arg_count++;    
    execute(arg_count, arg_values);
    arg_count = 0; 
  } while ( strcmp(arg_values[0], "exit") != 0 );

  return 1;
}
